@@ -18,12 +18,13 @@
 #
 
 __author__    = 'Thomas Perl <thp@gpodder.org>'
-__version__   = '0.16.1'
-__date__      = '2009-06-05'
+__version__   = '2.1'
+__date__      = '2009-12-12'
 __copyright__ = '© 2005-2009 Thomas Perl and the gPodder Team'
 __licence__   = 'GNU General Public License, version 3 or later'
 __url__       = 'http://gpodder.org/'
 
+import os
 import sys
 import platform
 import gettext
@@ -44,11 +45,18 @@ del feedparser
 # The User-Agent string for downloads
 user_agent = 'gPodder/%s (+%s)' % (__version__, __url__)
 
-# Interface type enums
-(CLI, GUI, MAEMO) = range(3)
-
 # Are we running in GUI, Maemo or console mode?
-interface = CLI
+class UI(object):
+    def __init__(self):
+        self.desktop = False
+        self.diablo = False
+        self.fremantle = False
+
+    @property
+    def maemo(self):
+        return self.diablo or self.fremantle
+
+ui = UI()
 
 # D-Bus specific interface names
 dbus_bus_name = 'org.godder'
@@ -88,6 +96,39 @@ del socket
 del SOCKET_TIMEOUT
 
 # Variables reserved for GUI-specific use (will be set accordingly)
-ui_folder = None
+ui_folders = []
+credits_file = None
 icon_file = None
 
+# Episode states used in the database
+STATE_NORMAL, STATE_DOWNLOADED, STATE_DELETED = range(3)
+
+# Default locations for configuration and data files
+home = os.path.expanduser(os.path.join('~', '.config', 'gpodder'))
+subscription_file = os.path.join(home, 'channels.opml')
+config_file = os.path.join(home, 'gpodder.conf')
+database_file = os.path.join(home, 'database.sqlite')
+
+# Plugins to load by default
+DEFAULT_PLUGINS = ['gpodder.soundcloud']
+
+def load_plugins():
+    """Load (non-essential) plugin modules
+
+    This loads a default set of plugins, but you can use
+    the environment variable "GPODDER_PLUGINS" to modify
+    the list of plugins."""
+    global DEFAULT_PLUGINS
+    PLUGINS = os.environ.get('GPODDER_PLUGINS', None)
+    if PLUGINS is None:
+        PLUGINS = DEFAULT_PLUGINS
+    else:
+        PLUGINS = PLUGINS.split()
+    import imp
+    for plugin in PLUGINS:
+        try:
+            __import__(plugin)
+            print >>sys.stderr, 'Plugin loaded:', plugin
+        except Exception, e:
+            print >>sys.stderr, 'Cannot load plugin: %s (%s)' % (plugin, e)
+
