@@ -32,13 +32,12 @@ are not tied to any specific part of gPodder.
 import gpodder
 from gpodder.liblogger import log
 
-import gtk
-import gobject
-
 import os
 import os.path
+import platform
 import glob
 import stat
+import shlex
 
 import re
 import subprocess
@@ -61,9 +60,11 @@ import feedparser
 import StringIO
 import xml.dom.minidom
 
+_ = gpodder.gettext
+
 
 # Try to detect OS encoding (by Leonid Ponomarev)
-if gpodder.interface == gpodder.MAEMO:
+if gpodder.ui.maemo:
     encoding = 'utf8'
 else:
     encoding = 'iso-8859-15'
@@ -80,14 +81,9 @@ else:
     log('is incorrect, please set your $LANG variable.')
 
 
-if gpodder.interface == gpodder.GUI:
-    ICON_UNPLAYED = gtk.STOCK_YES
-    ICON_LOCKED = 'emblem-nowrite'
-    ICON_MISSING = gtk.STOCK_STOP
-elif gpodder.interface == gpodder.MAEMO:
-    ICON_UNPLAYED = 'qgn_list_gene_favor'
-    ICON_LOCKED = 'qgn_indi_KeypadLk_lock'
-    ICON_MISSING = gtk.STOCK_STOP # FIXME!
+# Used by file_type_by_extension()
+_BUILTIN_FILE_TYPES = None
+
 
 def make_directory( path):
     """
@@ -117,13 +113,38 @@ def normalize_feed_url(url):
     Also supported are phobos.apple.com links (iTunes podcast)
     and itms:// links (iTunes podcast direct link).
 
+    >>> normalize_feed_url('itpc://example.org/podcast.rss')
+    'http://example.org/podcast.rss'
+
     If no URL scheme is defined (e.g. "curry.com"), we will
     simply assume the user intends to add a http:// feed.
-    """
 
+    >>> normalize_feed_url('curry.com')
+    'http://curry.com'
+
+    There are even some more shortcuts for advanced users
+    and lazy typists (see the source for details).
+
+    >>> normalize_feed_url('fb:43FPodcast')
+    'http://feeds2.feedburner.com/43FPodcast'
+    """
     if not url or len(url) < 8:
         return None
 
+    # This is a list of prefixes that you can use to minimize the amount of
+    # keystrokes that you have to use.
+    # Feel free to suggest other useful prefixes, and I'll add them here.
+    PREFIXES = {
+            'fb:': 'http://feeds2.feedburner.com/%s',
+            'yt:': 'http://www.youtube.com/rss/user/%s/videos.rss',
+            'sc:': 'http://soundcloud.com/%s',
+    }
+
+    for prefix, expansion in PREFIXES.iteritems():
+        if url.startswith(prefix):
+            url = expansion % (url[len(prefix):],)
+            break
+
     # Assume HTTP for URLs without scheme
     if not '://' in url:
         url = 'http://' + url
@@ -278,6 +299,21 @@ def file_modification_datetime(filename):
         return None
 
 
+def file_modification_timestamp(filename):
+    """
+    Returns the modification date of the specified file as a number
+    or -1 if the modification date cannot be determined.
+    """
+    if filename is None:
+        return -1
+    try:
+        s = os.stat(filename)
+        return s[stat.ST_MTIME]
+    except:
+        log('Cannot get modification timestamp for %s', filename)
+        return -1
+
+
 def file_age_in_days(filename):
     """
     Returns the age of the specified filename in days or
@@ -298,9 +334,9 @@ def file_age_to_string(days):
     >>> file_age_to_string(0)
     ''
     >>> file_age_to_string(1)
-    'one day ago'
+    u'one day ago'
     >>> file_age_to_string(2)
-    '2 days ago'
+    u'2 days ago'
     """
     if days == 1:
         return _('one day ago')
@@ -310,6 +346,27 @@ def file_age_to_string(days):
         return ''
 
 
+def get_free_disk_space_win32(path):
+    """
+    Win32-specific code to determine the free disk space remaining
+    for a given path. Uses code from:
+
+    http://mail.python.org/pipermail/python-list/2003-May/203223.html
+    """
+
+    drive, tail = os.path.splitdrive(path)
+
+    try:
+        import win32file
+        userFree, userTotal, freeOnDisk = win32file.GetDiskFreeSpaceEx(drive)
+        return userFree
+    except ImportError:
+        log('Warning: Running on Win32 but win32api/win32file not installed.')
+
+    # Cannot determine free disk space
+    return 0
+
+
 def get_free_disk_space(path):
     """
     Calculates the free disk space available to the current user
@@ -322,6 +379,9 @@ def get_free_disk_space(path):
     if not os.path.exists(path):
         return 0
 
+    if gpodder.win32:
+        return get_free_disk_space_win32(path)
+
     s = os.statvfs(path)
 
     return s.f_bavail * s.f_bsize
@@ -361,12 +421,17 @@ def format_date(timestamp):
         log('Warning: Cannot convert "%s" to date.', timestamp, traceback=True)
         return None
 
+    try:
+        timestamp = datetime.datetime.fromtimestamp(timestamp)
+    except:
+        return None
+
     if diff < 7:
         # Weekday name
-        return str(datetime.datetime.fromtimestamp(timestamp).strftime('%A'))
+        return str(timestamp.strftime('%A'))
     else:
         # Locale's appropriate date representation
-        return str(datetime.datetime.fromtimestamp(timestamp).strftime('%x'))
+        return str(timestamp.strftime('%x'))
 
 
 def format_filesize(bytesize, use_si_units=False, digits=2):
@@ -541,99 +606,60 @@ def filename_from_url(url):
     return ( filename, extension.lower() )
 
 
-def file_type_by_extension( extension):
+def file_type_by_extension(extension):
     """
     Tries to guess the file type by looking up the filename 
     extension from a table of known file types. Will return 
-    the type as string ("audio" or "video") or
-    None if the file type cannot be determined.
-    """
-    types = {
-            'audio': [ 'mp3', 'ogg', 'wav', 'wma', 'aac', 'm4a', 'm4b' ],
-            'video': [ 'mp4', 'avi', 'mpg', 'mpeg', 'm4v', 'mov', 'divx', 'flv', 'wmv', '3gp' ],
-    }
+    "audio", "video" or None.
 
-    if extension == '':
+    >>> file_type_by_extension('.aif')
+    'audio'
+    >>> file_type_by_extension('.3GP')
+    'video'
+    >>> file_type_by_extension('.txt') is None
+    True
+    >>> file_type_by_extension(None) is None
+    True
+    >>> file_type_by_extension('ogg')
+    Traceback (most recent call last):
+      ...
+    ValueError: Extension does not start with a dot: ogg
+    """
+    if not extension:
         return None
 
-    if extension[0] == '.':
-        extension = extension[1:]
+    if not extension.startswith('.'):
+        raise ValueError('Extension does not start with a dot: %s' % extension)
+
+    global _BUILTIN_FILE_TYPES
+    if _BUILTIN_FILE_TYPES is None:
+        # List all types that are not in the default mimetypes.types_map
+        # (even if they might be detected by mimetypes.guess_type)
+        # For OGG, see http://wiki.xiph.org/MIME_Types_and_File_Extensions
+        audio_types = ('.ogg', '.oga', '.spx', '.flac', '.axa', \
+                       '.aac', '.m4a', '.m4b', '.wma')
+        video_types = ('.ogv', '.axv', '.mp4', \
+                       '.mkv', '.m4v', '.divx', '.flv', '.wmv', '.3gp')
+        _BUILTIN_FILE_TYPES = {}
+        _BUILTIN_FILE_TYPES.update((ext, 'audio') for ext in audio_types)
+        _BUILTIN_FILE_TYPES.update((ext, 'video') for ext in video_types)
 
     extension = extension.lower()
 
-    for type in types:
-        if extension in types[type]:
-            return type
+    if extension in _BUILTIN_FILE_TYPES:
+        return _BUILTIN_FILE_TYPES[extension]
+
+    # Need to prepend something to the extension, so guess_type works
+    type, encoding = mimetypes.guess_type('file'+extension)
+
+    if type is not None and '/' in type:
+        filetype, rest = type.split('/', 1)
+        if filetype in ('audio', 'video'):
+            return filetype
     
     return None
 
 
-def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, add_missing=False, icon_cache=None, icon_size=32):
-    """
-    Loads an icon from the current icon theme at the specified
-    size, suitable for display in a gtk.TreeView.
-
-    Optionally adds a green bullet (the GTK Stock "Yes" icon)
-    to the Pixbuf returned. Also, a padlock icon can be added.
-
-    If an icon_cache parameter is supplied, it has to be a
-    dictionary and will be used to store generated icons. 
-
-    On subsequent calls, icons will be loaded from cache if 
-    the cache is supplied again and the icon is found in 
-    the cache.
-    """
-    global ICON_UNPLAYED, ICON_LOCKED, ICON_MISSING
-
-    if icon_cache is not None and (icon_name,add_bullet,add_padlock,icon_size) in icon_cache:
-        return icon_cache[(icon_name,add_bullet,add_padlock,icon_size)]
-    
-    icon_theme = gtk.icon_theme_get_default()
-
-    try:
-        icon = icon_theme.load_icon(icon_name, icon_size, 0)
-    except:
-        log( '(get_tree_icon) Warning: Cannot load icon with name "%s", will use  default icon.', icon_name)
-        icon = icon_theme.load_icon(gtk.STOCK_DIALOG_QUESTION, icon_size, 0)
-
-    if icon and (add_bullet or add_padlock or add_missing):
-        # We'll modify the icon, so use .copy()
-        if add_missing:
-            log('lalala')
-            try:
-                icon = icon.copy()
-                emblem = icon_theme.load_icon(ICON_MISSING, int(float(icon_size)*1.2/3.0), 0)
-                (width, height) = (emblem.get_width(), emblem.get_height())
-                xpos = icon.get_width() - width
-                ypos = icon.get_height() - height
-                emblem.composite(icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
-            except:
-                log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
-        elif add_bullet:
-            try:
-                icon = icon.copy()
-                emblem = icon_theme.load_icon(ICON_UNPLAYED, int(float(icon_size)*1.2/3.0), 0)
-                (width, height) = (emblem.get_width(), emblem.get_height())
-                xpos = icon.get_width() - width
-                ypos = icon.get_height() - height
-                emblem.composite(icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
-            except:
-                log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
-        if add_padlock:
-            try:
-                icon = icon.copy()
-                emblem = icon_theme.load_icon(ICON_LOCKED, int(float(icon_size)/2.0), 0)
-                (width, height) = (emblem.get_width(), emblem.get_height())
-                emblem.composite(icon, 0, 0, width, height, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
-            except:
-                log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
-
-    if icon_cache is not None:
-        icon_cache[(icon_name,add_bullet,add_padlock,icon_size)] = icon
-
-    return icon
-
-
 def get_first_line( s):
     """
     Returns only the first line of a string, stripped so
@@ -674,7 +700,7 @@ def object_string_formatter( s, **kwargs):
     return result
 
 
-def format_desktop_command( command, filename):
+def format_desktop_command(command, filenames):
     """
     Formats a command template from the "Exec=" line of a .desktop
     file to a string that can be invoked in a shell.
@@ -683,32 +709,135 @@ def format_desktop_command( command, filename):
     appends the filename as first parameter of the command.
 
     See http://standards.freedesktop.org/desktop-entry-spec/1.0/ar01s06.html
+
+    Returns a list of commands to execute, either one for
+    each filename if the application does not support multiple
+    file names or one for all filenames (%U, %F or unknown).
     """
-    if '://' in filename:
-        filename_url = filename
+    command = shlex.split(command)
+
+    command_before = command
+    command_after = []
+    multiple_arguments = True
+    for fieldcode in ('%U', '%F', '%u', '%f'):
+        if fieldcode in command:
+            command_before = command[:command.index(fieldcode)]
+            command_after = command[command.index(fieldcode)+1:]
+            multiple_arguments = fieldcode in ('%U', '%F')
+            break
+
+    if multiple_arguments:
+        return [command_before + filenames + command_after]
+
+    commands = []
+    for filename in filenames:
+        commands.append(command_before+[filename]+command_after)
+
+    return commands
+
+def url_strip_authentication(url):
+    """
+    Strips authentication data from an URL. Returns the URL with
+    the authentication data removed from it.
+
+    >>> url_strip_authentication('https://host.com/')
+    'https://host.com/'
+    >>> url_strip_authentication('telnet://foo:bar@host.com/')
+    'telnet://host.com/'
+    >>> url_strip_authentication('ftp://billy@example.org')
+    'ftp://example.org'
+    >>> url_strip_authentication('ftp://billy:@example.org')
+    'ftp://example.org'
+    >>> url_strip_authentication('http://aa:bc@localhost/x')
+    'http://localhost/x'
+    >>> url_strip_authentication('http://i%2Fo:P%40ss%3A@blubb.lan/u.html')
+    'http://blubb.lan/u.html'
+    >>> url_strip_authentication('http://c:d@x.org/')
+    'http://x.org/'
+    >>> url_strip_authentication('http://P%40%3A:i%2F@cx.lan')
+    'http://cx.lan'
+    """
+    url_parts = list(urlparse.urlsplit(url))
+    # url_parts[1] is the HOST part of the URL
+
+    # Remove existing authentication data
+    if '@' in url_parts[1]:
+        url_parts[1] = url_parts[1].split('@', 2)[1]
+
+    return urlparse.urlunsplit(url_parts)
+
+
+def url_add_authentication(url, username, password):
+    """
+    Adds authentication data (username, password) to a given
+    URL in order to construct an authenticated URL.
+
+    >>> url_add_authentication('https://host.com/', '', None)
+    'https://host.com/'
+    >>> url_add_authentication('http://example.org/', None, None)
+    'http://example.org/'
+    >>> url_add_authentication('telnet://host.com/', 'foo', 'bar')
+    'telnet://foo:bar@host.com/'
+    >>> url_add_authentication('ftp://example.org', 'billy', None)
+    'ftp://billy@example.org'
+    >>> url_add_authentication('ftp://example.org', 'billy', '')
+    'ftp://billy:@example.org'
+    >>> url_add_authentication('http://localhost/x', 'aa', 'bc')
+    'http://aa:bc@localhost/x'
+    >>> url_add_authentication('http://blubb.lan/u.html', 'i/o', 'P@ss:')
+    'http://i%2Fo:P%40ss%3A@blubb.lan/u.html'
+    >>> url_add_authentication('http://a:b@x.org/', 'c', 'd')
+    'http://c:d@x.org/'
+    >>> url_add_authentication('http://i%2F:P%40%3A@cx.lan', 'P@:', 'i/')
+    'http://P%40%3A:i%2F@cx.lan'
+    """
+    if username is None or username == '':
+        return url
+
+    username = urllib.quote_plus(username)
+
+    if password is not None:
+        password = urllib.quote_plus(password)
+        auth_string = ':'.join((username, password))
     else:
-        filename_url = 'file://%s' % filename
+        auth_string = username
 
-    items = {
-            '%U': filename_url,
-            '%u': filename_url,
-            '%F': filename,
-            '%f': filename,
-    }
+    url = url_strip_authentication(url)
 
-    for key, value in items.items():
-        if command.find( key) >= 0:
-            return command.replace( key, '"%s"' % value)
+    url_parts = list(urlparse.urlsplit(url))
+    # url_parts[1] is the HOST part of the URL
+    url_parts[1] = '@'.join((auth_string, url_parts[1]))
 
-    return '%s "%s"' % ( command, filename )
+    return urlparse.urlunsplit(url_parts)
 
 
 def get_real_url(url):
     """
     Gets the real URL of a file and resolves all redirects.
     """
-    return urllib.urlopen(url).geturl()
+    try:
+        username, password = username_password_from_url(url)
+        if username or password:
+            url = url_strip_authentication(url)
+            log('url=%s, username=%s, password=%s', url, username, password)
+            password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
+            password_mgr.add_password(None, url, username, password)
+            handler = urllib2.HTTPBasicAuthHandler(password_mgr)
+            opener = urllib2.build_opener(handler)
+            return opener.open(url).geturl()
+        else:
+            return urlopen(url).geturl()
+    except:
+        log('Error getting real url for %s', url, traceback=True)
+        return url
 
+def urlopen(url):
+    """
+    An URL opener with the User-agent set to gPodder (with version)
+    """
+    headers = {'User-agent': gpodder.user_agent}
+    request = urllib2.Request(url, headers=headers)
+    return urllib2.urlopen(request)
 
 def find_command( command):
     """
@@ -820,7 +949,8 @@ def idle_add(func, *args):
     call the function later - this is needed for
     threads to be able to modify GTK+ widget data.
     """
-    if gpodder.interface in (gpodder.GUI, gpodder.MAEMO):
+    if gpodder.ui.desktop or gpodder.ui.maemo:
+        import gobject
         def x(f, *a):
             f(*a)
             return False
@@ -830,61 +960,21 @@ def idle_add(func, *args):
         func(*args)
 
 
-def discover_bluetooth_devices():
-    """
-    This is a generator function that returns
-    (address, name) tuples of all nearby bluetooth
-    devices found.
-
-    If the user has python-bluez installed, it will
-    be used. If not, we're trying to use "hcitool".
-
-    If neither python-bluez or hcitool are available,
-    this function is the empty generator.
-    """
-    try:
-        # If the user has python-bluez installed
-        import bluetooth
-        log('Using python-bluez to find nearby bluetooth devices')
-        for name, addr in bluetooth.discover_devices(lookup_names=True):
-            yield (name, addr)
-    except:
-        if find_command('hcitool') is not None:
-            log('Using hcitool to find nearby bluetooth devices')
-            # If the user has "hcitool" installed
-            p = subprocess.Popen(['hcitool', 'scan'], stdout=subprocess.PIPE)
-            for line in p.stdout:
-                match = re.match('^\t([^\t]+)\t([^\t]+)\n$', line)
-                if match is not None:
-                    (addr, name) = match.groups()
-                    yield (name, addr)
-        else:
-            log('Cannot find either python-bluez or hcitool - no bluetooth?')
-            return # <= empty generator
-
-
 def bluetooth_available():
     """
     Returns True or False depending on the availability
     of bluetooth functionality on the system.
     """
-    if find_command('bluetooth-sendto'):
-        return True
-    elif find_command('gnome-obex-send'):
+    if find_command('bluetooth-sendto') or \
+            find_command('gnome-obex-send'):
         return True
     else:
         return False
 
 
-def bluetooth_send_file(filename, device=None, callback_finished=None):
+def bluetooth_send_file(filename):
     """
-    Sends a file via bluetooth using gnome-obex send.
-    Optional parameter device is the bluetooth address
-    of the device; optional parameter callback_finished
-    is a callback function that will be called when the
-    sending process has finished - it gets one parameter
-    that is either True (when sending succeeded) or False
-    when there was some error.
+    Sends a file via bluetooth.
 
     This function tries to use "bluetooth-sendto", and if
     it is not available, it also tries "gnome-obex-send".
@@ -893,23 +983,14 @@ def bluetooth_send_file(filename, device=None, callback_finished=None):
 
     if find_command('bluetooth-sendto'):
         command_line = ['bluetooth-sendto']
-        if device is not None:
-            command_line.append('--device=%s' % device)
     elif find_command('gnome-obex-send'):
         command_line = ['gnome-obex-send']
-        if device is not None:
-            command_line += ['--dest', device]
 
     if command_line is not None:
         command_line.append(filename)
-        result = (subprocess.Popen(command_line).wait() == 0)
-        if callback_finished is not None:
-            callback_finished(result)
-        return result
+        return (subprocess.Popen(command_line).wait() == 0)
     else:
         log('Cannot send file. Please install "bluetooth-sendto" or "gnome-obex-send".')
-        if callback_finished is not None:
-            callback_finished(False)
         return False
         
         
@@ -919,11 +1000,11 @@ def format_seconds_to_hour_min_sec(seconds):
     human-readable string (duration).
 
     >>> format_seconds_to_hour_min_sec(3834)
-    '1 hour, 3 minutes and 54 seconds'
+    u'1 hour, 3 minutes and 54 seconds'
     >>> format_seconds_to_hour_min_sec(3600)
-    '1 hour'
+    u'1 hour'
     >>> format_seconds_to_hour_min_sec(62)
-    '1 minute and 2 seconds'
+    u'1 minute and 2 seconds'
     """
 
     if seconds < 1:
@@ -1016,12 +1097,30 @@ def get_episode_info_from_url(url, proxy=None):
 def gui_open(filename):
     """
     Open a file or folder with the default application set
-    by the Desktop environment. This uses "xdg-open".
+    by the Desktop environment. This uses "xdg-open" on all
+    systems with a few exceptions:
+
+       on Win32, os.startfile() is used
+       on Maemo, osso is used to communicate with Nokia Media Player
     """
     try:
-        subprocess.Popen(['xdg-open', filename])
-        # FIXME: Win32-specific "open" code needed here
-        # as fallback when xdg-open not available
+        if gpodder.ui.maemo:
+            try:
+                import osso
+            except ImportError, ie:
+                log('Cannot import osso module on maemo.')
+                return False
+
+            log('Using Nokia Media Player to open %s', filename)
+            context = osso.Context('gPodder', gpodder.__version__, False)
+            filename = filename.encode('utf-8')
+            rpc = osso.Rpc(context)
+            service, path = 'com.nokia.mediaplayer', '/com/nokia/mediaplayer'
+            rpc.rpc_run(service, path, service, 'mime_open', (filename,))
+        elif gpodder.win32:
+            os.startfile(filename)
+        else:
+            subprocess.Popen(['xdg-open', filename])
         return True
     except:
         log('Cannot open file/folder: "%s"', filename, traceback=True)
@@ -1034,7 +1133,15 @@ def open_website(url):
     browser. This uses Python's "webbrowser" module, so
     make sure your system is set up correctly.
     """
-    threading.Thread(target=webbrowser.open, args=(url,)).start()
+    if gpodder.ui.maemo:
+        import osso
+        context = osso.Context('gPodder', gpodder.__version__, False)
+        rpc = osso.Rpc(context)
+        rpc.rpc_run_with_defaults('osso_browser', \
+                                  'open_new_window', \
+                                  (url,))
+    else:
+        threading.Thread(target=webbrowser.open, args=(url,)).start()
 
 def sanitize_encoding(filename):
     r"""
@@ -1105,7 +1212,7 @@ def find_mount_point(directory):
       ...
     ValueError: Directory names should be of type str.
 
-    >>> from minimock import mock
+    >>> from minimock import mock, restore
     >>> mocked_mntpoints = ('/', '/home', '/media/usbdisk', '/media/cdrom')
     >>> mock('os.path.ismount', returns_func=lambda x: x in mocked_mntpoints)
     >>>
@@ -1142,6 +1249,7 @@ def find_mount_point(directory):
     Called os.path.ismount('/media/usbdisk/blubb')
     Called os.path.ismount('/media/usbdisk')
     '/media/usbdisk'
+    >>> restore()
     """
     if isinstance(directory, unicode):
         # We do not accept unicode strings, because they could fail when
@@ -1163,46 +1271,6 @@ def find_mount_point(directory):
     return '/'
 
 
-def resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, key=None, cache=None):
-    """
-    Resizes a GTK Pixbuf but keeps its aspect ratio.
-    
-    Returns None if the pixbuf does not need to be
-    resized or the newly resized pixbuf if it does.
-
-    The optional parameter "key" is used to identify
-    the image in the "cache", which is a dict-object
-    that holds already-resized pixbufs to access.
-    """
-    changed = False
-
-    if cache is not None:
-        if (key, max_width, max_height) in cache:
-            return cache[(key, max_width, max_height)]
-
-    # Resize if too wide
-    if pixbuf.get_width() > max_width:
-        f = float(max_width)/pixbuf.get_width()
-        (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
-        pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
-        changed = True
-
-    # Resize if too high
-    if pixbuf.get_height() > max_height:
-        f = float(max_height)/pixbuf.get_height()
-        (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
-        pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
-        changed = True
-
-    if changed:
-        result = pixbuf
-        if cache is not None:
-            cache[(key, max_width, max_height)] = result
-    else:
-        result = None
-
-    return result
-
 # matches http:// and ftp:// and mailto://
 protocolPattern = re.compile(r'^\w+://')
 
@@ -1257,3 +1325,37 @@ def relpath(p1, p2):
 
     return os.path.join(*p)
 
+
+def run_external_command(command_line):
+    """
+    This is the function that will be called in a separate
+    thread that will call an external command (specified by
+    command_line). In case of problem (i.e. the command has
+    not been found or there has been another error), we will
+    call the notification function with two arguments - the
+    first being the error message and the second being the
+    title to be used for the error message.
+
+    >>> from minimock import mock, Mock, restore
+    >>> mock('subprocess.Popen', returns=Mock('subprocess.Popen'))
+    >>> run_external_command('testprogramm')
+    Called subprocess.Popen('testprogramm', shell=True)
+    Called subprocess.Popen.wait()
+    >>> restore()
+    """
+
+    def open_process(command_line):
+        log('Running external command: %s', command_line)
+        p = subprocess.Popen(command_line, shell=True)
+        result = p.wait()
+        if result == 127:
+            log('Command not found: %s', command_line)
+        elif result == 126:
+            log('Command permission denied: %s', command_line)
+        elif result > 0:
+            log('Command returned an error (%d): %s', result, command_line)
+        else:
+            log('Command finished successfully: %s', command_line)
+
+    threading.Thread(target=open_process, args=(command_line,)).start()
+
